/*--------------------------Probability_Gates_110.c  -----------------------------------
 * 
 * 
 * Tested and running on ATMega 328p with 8MHz internal clock
 * 
 * 
 * Fuse settings:
 * avrdude -p m328p -c avrisp2 -U lfuse:w:0xe2:m 
 * avrdude -p m328p -c avrisp2 -U hfuse:w:0xd9:m 
 * avrdude -p m328p -c avrisp2 -U efuse:w:0x07:m 
 * 
 * 
 * avrdude -p m328p -c avrisp2                      ; console command
 * avrdude: Device signature = 0x1e950f             ; default output new unused device 
 * avrdude: safemode: Fuses OK (E:07, H:D9, L:62)   ; default output new unused device
 * 
 * 
 * compile:
 * avr-gcc -Wall -Os -mmcu=atmega328p Probalistic_Gates_110.c -o Probalistic_Gates_110.elf
 * avr-objcopy -O ihex -j .text -j .data Probalistic_Gates_110.elf Probalistic_Gates_110.hex
 * avrdude -p m328p -c avrisp2 -U flash:w:Probalistic_Gates_110.hex
 * 
 * 
 * Used hardware:
 * PCB for Clock_Divider 120-B and 101-A
 * 
 * 
 * ---------------------------------------------------------------------------------------
 *
 * 20250106 First run
 * 
 * 20250320 
 * Added output_register to remove calculation from ISR routine
 * Added variables for probability values
 * 
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdbool.h>


volatile uint8_t probal;				// Holds the (pseudo)random number
uint8_t output_register;				// Holds the output value

//INT0 interrupt 
ISR(INT0_vect )																					// Clock in goes high
	{ 
		output_register = output_register | 0b00000001;			// Clock output / LED
		PORTB = output_register;														// Output random gates on PORT B
	} 

ISR(INT1_vect)						// Clock in goes low
 {
	PORTB = 0b00000000;			// All outputs to zero
 }
    
volatile uint8_t set_random()						// Generate pseudo random number 
{
	return(rand() & 0xFF);								// Reduce the range of rand() to 0..255
}

int main (void)
{
	bool stop = false;										// Holds state of Stop/Start input
	uint8_t probalistic_1 = 25;						// probability distribution values
	uint8_t probalistic_2 = 50;						// Change those values if you want
	uint8_t probalistic_3 = 75;						// other distribution
	uint8_t probalistic_4 = 100;
	uint8_t probalistic_5 = 125;
	uint8_t probalistic_6 = 150;
	uint8_t probalistic_7 = 175;
	
	DDRC &=~ (1 << PC0);                  // set PC0 as input; Start/Stop
  DDRD &=~ (1 << PD3);									// set PD3 as input; INT0  
  DDRD &=~ (1 << PD2);                  // set PD2 as input; INT1  
  DDRB = 0b11111111;                    // PORTB output
  EIMSK |= (1<<INT0);										// enable INT0
  EIMSK |= (1<<INT1);										// enable INT1
  EICRA |= (1<<ISC00)|(1<<ISC01);       // INT0 - Rising edge on INT0 generates an interrupt request
  EICRA |= (1<<ISC11);                  // INT1 - Falling edge of INT1 generates interrupt request
  PORTB = 0b00000000;                   // All outputs to zero 
  sei();																// Interrupts activate
  probal = set_random();								// first set probal
  
  while (1)															// for ever
    {				
        stop = (PINC & (1<< PC0));			// Read Stop/Start input
        while (stop)										// Stop output when high
        {
            cli();											// Disable interrupts
            PORTB = 0b00000000;					// All outputs to zero 
            stop = (PINC & (1<< PC0));	// Read Stop/Start input
						probal = set_random();			// set new random to probal
        }
        sei();													// Interrupts (re)activate
				probal = set_random();					// set new random value to probal
				output_register = 0;						// Delete previous output value
        if (probal <= probalistic_1)		// Output 1
        {
            output_register = output_register | 0b00000010;
        }
        
        if (probal <= probalistic_2)		// Output 2
        {
            output_register =output_register | 0b00000100;
        } 
        
        if (probal <= probalistic_3)		// Output 2
        {
            output_register =output_register | 0b00001000;
        } 
        
        if (probal <= probalistic_4)		// Output 2
        {
            output_register =output_register | 0b00010000;
        } 
        
         if (probal <= probalistic_5)		// Output 2
        {
            output_register =output_register | 0b00100000;
        } 
       
         if (probal <= probalistic_6)		// Output 2
        {
            output_register =output_register | 0b01000000;
        } 
        
         if (probal <= probalistic_7)		// Output 2
        {
            output_register =output_register | 0b10000000;
        } 
    }
	return(0);														// You'll never reach this line
}
